'use strict';

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _generate = require('../js/generate.js');

var _generate2 = _interopRequireDefault(_generate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var WHITELIST = [
// 'additionalItems.json',
'additionalProperties.json', 'allOf.json', 'anyOf.json', 'default.json',
// 'definitions.json',
// 'dependencies.json',
'enum.json', 'items.json', 'maxItems.json', 'maxLength.json', 'maxProperties.json', 'maximum.json', 'minItems.json', 'minLength.json', 'minProperties.json', 'minimum.json',
// 'multipleOf.json',
'not.json', 'oneOf.json', 'optional', 'pattern.json', 'patternProperties.json', 'properties.json', 'ref.json',
// 'refRemote.json',
'required.json', 'type.json'];


var singleTest = function singleTest(test, code, validator) {
  var description = test.description,
      data = test.data,
      valid = test.valid;

  it(description, function () {
    expect(validator(data) === null).toBe(valid);
  });
};

var testGroup = function testGroup(group) {
  var description = group.description,
      schema = group.schema,
      tests = group.tests;

  var code = (0, _generate2.default)(schema);
  it('matches snapshot', function () {
    expect(code).toMatchSnapshot();
  });
  var validator = eval(code).root; // eslint-disable-line no-eval
  describe(description, function () {
    tests.forEach(function (test) {
      return singleTest(test, code, validator);
    });
  });
};

var testFile = function testFile(testsPath, fileName) {
  // $FlowFixMe Dynamic require for DRY
  var groups = require(_path2.default.join(testsPath, fileName));
  if (WHITELIST.indexOf(fileName) >= 0) {
    describe(fileName, function () {
      groups.forEach(testGroup);
    });
  }
};

describe('JSON Schema test suite', function () {
  var testsPath = _path2.default.join(__dirname, '../../', 'JSON-Schema-Test-Suite/tests/draft4/');
  _fs2.default.readdirSync(testsPath).forEach(function (fileName) {
    if (fileName.match(/.*\.json$/)) {
      testFile(testsPath, fileName);
    }
  });
});