'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _util = require('../../util.js');

var _util2 = _interopRequireDefault(_util);

var _ast = require('../jsast/ast.js');

var _ast2 = _interopRequireDefault(_ast);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Generates all checks for
//   - minimum/maximum (number)
//   - minLength/maxLength (string)
//   - minItems/maxItems (array)
//   - minProperties/maxProperties (object)
var comparison = function comparison(symbol, comparator, base, error) {
  if (base !== undefined) {
    return _ast2.default.If(_ast2.default.Binop.Any(comparator)(symbol, _lodash2.default.toString(base)), error);
  } else {
    return _ast2.default.Empty;
  }
};


var comparisons = function comparisons(schema, symbol, context) {
  var error = context.error();
  return _ast2.default.Body(_util2.default.typeCheck('number', symbol, _ast2.default.Body(comparison(symbol, schema.exclusiveMinimum ? '<=' : '<', schema.minimum, error), comparison(symbol, schema.exclusiveMaximum ? '>=' : '>', schema.maximum, error))), _util2.default.typeCheck('string', symbol, _ast2.default.Body(comparison(symbol + '.length', '<', schema.minLength, error), comparison(symbol + '.length', '>', schema.maxLength, error))), _util2.default.typeCheck('array', symbol, _ast2.default.Body(comparison(symbol + '.length', '<', schema.minItems, error), comparison(symbol + '.length', '>', schema.maxItems, error))), _util2.default.typeCheck('object', symbol, _ast2.default.Body(comparison('Object.keys(' + symbol + ').length', '<', schema.minProperties, error), comparison('Object.keys(' + symbol + ').length', '>', schema.maxProperties, error))));
};

exports.default = comparisons;