'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _util = require('../../util.js');

var _util2 = _interopRequireDefault(_util);

var _ast = require('../jsast/ast.js');

var _ast2 = _interopRequireDefault(_ast);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var additionalChecks = function additionalChecks(properties, patternProperties, additionalProperties, keySym, valSym, context) {
  // This generates a block of code like
  //   if (key === "key1" && error(check1(data))) { (error) }
  //   if (key === "key2" && error(check2(data))) { (error) }
  //   if (key.match(/pattern3/) && error(check3(data))) { (error) }
  //   if (key.match(/pattern4/) && error(check4(data))) { (error) }
  //   if (!hit && error(additionalCheck(data))) { (error) }
  var propertyChecks = _lodash2.default.map(properties, function (subSchema, key) {
    return {
      predicate: keySym + ' === "' + key + '"',
      subSchema: subSchema
    };
  });
  var patternChecks = _lodash2.default.map(patternProperties, function (subSchema, pattern) {
    return {
      predicate: keySym + '.match(/' + pattern + '/)',
      subSchema: subSchema
    };
  });
  var allChecks = [].concat(_toConsumableArray(propertyChecks), _toConsumableArray(patternChecks));

  if (additionalProperties === undefined) {
    // There are properties/patternProperties, but no additionalProperties. In
    // this case, we don't need to mark non-matches
    return _ast2.default.Body.apply(_ast2.default, _toConsumableArray(_lodash2.default.map(allChecks, function (_ref) {
      var predicate = _ref.predicate,
          subSchema = _ref.subSchema;

      var fnSym = context.symbolForSchema(subSchema);
      return _ast2.default.If(predicate, _ast2.default.If(_ast2.default.Binop.Neq(_ast2.default.Call(fnSym, valSym), 'null'), context.error()));
    })));
  } else if (allChecks.length === 0) {
    // There are no properties nor patternProperties, but additionalProperties.
    // In this case, we can always run additionalProperties checks.
    if (additionalProperties === false) {
      return context.error();
    } else {
      var fnSym = context.symbolForSchema(additionalProperties);
      return _ast2.default.If(_ast2.default.Binop.Neq(_ast2.default.Call(fnSym, valSym), 'null'), context.error());
    }
  } else {
    // There are both properties/patternProperties and additionalProperties. In
    // this case, we need to check if we've hit a property/patternProperty to
    // decide whether or not to check additionalProperties.
    var hitSym = context.gensym();
    var checks = _lodash2.default.map(allChecks, function (_ref2) {
      var predicate = _ref2.predicate,
          subSchema = _ref2.subSchema;

      var fnSym = context.symbolForSchema(subSchema);
      return _ast2.default.If(predicate, _ast2.default.If(_ast2.default.Binop.Neq(_ast2.default.Call(fnSym, valSym), 'null'), context.error(), _ast2.default.Assignment(hitSym, _ast2.default.Literal('true'))));
    });
    var additionalCheck = _ast2.default.If(_ast2.default.Binop.Eq(hitSym, 'false'), additionalProperties === false ? context.error() : _ast2.default.If(context.symbolForSchema(additionalProperties) + '(' + valSym + ') !== null', context.error()));
    return _ast2.default.Body.apply(_ast2.default, [_ast2.default.Assignment(hitSym, _ast2.default.Literal('false'))].concat(_toConsumableArray(checks), [additionalCheck]));
  }
};

var properties = function properties(schema, symbol, context) {
  if (schema.patternProperties || schema.additionalProperties !== undefined) {
    // Need to loop through all properties to check. We'll generate a loop:
    //   for (var key in json) {
    //     var val = json[key];
    //     var hit = false;
    //     if (key === property1) { ... }
    //     if (key === property2) { ... }
    //     if (key.match(pattern1)) { ... }
    //     if (key.match(pattern2)) { ... }
    //     if (!hit) { ... }
    //   }
    var keySym = context.gensym();
    var valSym = context.gensym();

    var loop = _ast2.default.ForIn(keySym, _ast2.default.Literal(symbol), _ast2.default.Body(_ast2.default.Assignment(valSym, _ast2.default.Literal(symbol + '[' + keySym + ']')), additionalChecks(schema.properties, schema.patternProperties, schema.additionalProperties, keySym, valSym, context)));
    return _util2.default.typeCheck('object', symbol, loop);
  } else if (schema.properties) {
    // Static list of properties to check
    var checks = _ast2.default.Body.apply(_ast2.default, _toConsumableArray(_lodash2.default.flatMap(schema.properties, function (subSchema, key) {
      var fnSym = context.symbolForSchema(subSchema);
      var sym = context.gensym();
      return _ast2.default.Body(_ast2.default.Assignment(sym, _ast2.default.Literal(symbol + '.' + key)), _ast2.default.If(_ast2.default.Binop.Neq(sym, 'undefined'), _ast2.default.If(_ast2.default.Binop.Neq(_ast2.default.Call(fnSym, sym), 'null'), context.error())));
    })));
    return _util2.default.typeCheck('object', symbol, checks);
  } else {
    return _ast2.default.Empty;
  }
};

exports.default = properties;