'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _root = require('./checks/root.js');

var _root2 = _interopRequireDefault(_root);

var _ast = require('./jsast/ast.js');

var _ast2 = _interopRequireDefault(_ast);

var _simplify = require('./jsast/simplify.js');

var _simplify2 = _interopRequireDefault(_simplify);

var _render = require('./jsast/render.js');

var _render2 = _interopRequireDefault(_render);

var _uniqFuncs = require('./jsast/uniq-funcs.js');

var _uniqFuncs2 = _interopRequireDefault(_uniqFuncs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var gengensym = function gengensym() {
  var cache = {};
  return function () {
    var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'v';

    cache[prefix] = cache[prefix] || 0;
    return '' + prefix + cache[prefix]++;
  };
};

// Given a root schema and a shape (map of schemas), returns a string which, if
// run in the body of a function, returns an object of the same shape whose
// values are validators for the schemas.
//
// If no shape is passed, {root} is used.
var generateValidator = function generateValidator(schema) {
  var shape = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { root: schema };

  var gensym = gengensym();

  var cache = new WeakMap();
  var schemas = [];

  var symbolForSchema = function symbolForSchema(schm) {
    if (!cache.has(schm)) {
      var match = _lodash2.default.find(schemas, function (s) {
        return _lodash2.default.isEqual(s, schm);
      });
      if (match) {
        cache.set(schm, cache.get(match));
      } else {
        cache.set(schm, gensym('f'));
        schemas.push(schm);
      }
    }
    return cache.get(schm);
  };

  var makeContext = function makeContext() {
    return {
      gensym: gengensym(),
      error: function error() {
        return _ast2.default.Return('"error"');
      },
      symbolForSchema: symbolForSchema,
      rootSchema: schema
    };
  };

  var baseSchemas = _lodash2.default.map(shape, function (subSchema) {
    return (0, _root2.default)(subSchema, makeContext());
  });
  var results = _lodash2.default.keyBy(baseSchemas, function (f) {
    return f.name.value;
  });
  var i = 1;
  while (i < schemas.length) {
    var next = (0, _root2.default)(schemas[i], makeContext());
    results[next.name.value] = next;
    i++;
  }

  // TODO: Fix flow unhelpful error
  var simplifiedResults = _lodash2.default.values(results).map(_simplify2.default);

  var schemaObject = _lodash2.default.mapValues(shape, function (subSchema) {
    return _ast2.default.Literal(symbolForSchema(subSchema));
  });
  var ast = _ast2.default.Body.apply(_ast2.default, _toConsumableArray(simplifiedResults).concat([_ast2.default.Return(_ast2.default.ObjectLiteral(schemaObject))]));
  var body = (0, _render2.default)((0, _simplify2.default)((0, _uniqFuncs2.default)((0, _simplify2.default)(ast))), 1);
  return ['(function() {', body, '})()'].join('\n');
};

exports.default = generateValidator;