'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

/* eslint-disable no-use-before-define */

// Statements
var Function1 = function Function1(name, argument, body) {
  return {
    type: 'function1',
    name: Literal(name),
    argument: Literal(argument),
    body: body
  };
};
var _Binop = function _Binop(comparator) {
  return function (left, right) {
    return {
      type: 'binop',
      comparator: comparator,
      left: typeof left === 'string' ? Literal(left) : left,
      right: typeof right === 'string' ? Literal(right) : right
    };
  };
};
var Assignment = function Assignment(variable, value) {
  return {
    type: 'assignment',
    variable: Literal(variable),
    value: value
  };
};
var If = function If(predicate, body) {
  var elseBody = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : Empty;

  return {
    type: 'if',
    predicate: typeof predicate === 'string' ? Literal(predicate) : predicate,
    body: typeof body === 'string' ? Literal(body) : body,
    elseBody: typeof elseBody === 'string' ? Literal(elseBody) : elseBody
  };
};
var Return = function Return(value) {
  return {
    type: 'return',
    value: typeof value === 'string' ? Literal(value) : value
  };
};
var Body = function Body() {
  for (var _len = arguments.length, body = Array(_len), _key = 0; _key < _len; _key++) {
    body[_key] = arguments[_key];
  }

  return { type: 'body', body: body };
};
var For = function For(init, condition, loop, body) {
  return { type: 'for', init: init, condition: condition, loop: loop, body: body };
};
var ForIn = function ForIn(variable, iterator, body) {
  return {
    type: 'forin',
    variable: Literal(variable),
    iterator: iterator,
    body: body
  };
};
var Empty = { type: 'empty' };
var Literal = function Literal(value) {
  if (typeof value === 'string') {
    return { type: 'literal', value: value };
  } else {
    return value;
  }
};
var Call = function Call(fn, arg) {
  return {
    type: 'call',
    fn: Literal(fn),
    arg: Literal(arg)
  };
};
var Not = function Not(child) {
  return {
    type: 'not',
    child: typeof child === 'string' ? Literal(child) : child
  };
};
var ObjectLiteral = function ObjectLiteral(object) {
  return {
    type: 'objectliteral',
    object: object
  };
};

exports.default = {
  Function1: Function1,
  Binop: {
    Eq: _Binop('==='),
    Neq: _Binop('!=='),
    And: _Binop('&&'),
    Or: _Binop('||'),
    Lt: _Binop('<'),
    Gt: _Binop('>'),
    Lte: _Binop('<='),
    Gte: _Binop('>='),
    Any: _Binop
  },
  Assignment: Assignment,
  If: If,
  Return: Return,
  Body: Body,
  For: For,
  ForIn: ForIn,
  Empty: Empty,
  Literal: Literal,
  Call: Call,
  Not: Not,
  ObjectLiteral: ObjectLiteral
};