'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _ast = require('./ast.js');

var _ast2 = _interopRequireDefault(_ast);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }
/* eslint-disable no-use-before-define */


var simplifyIf = function simplifyIf(ast) {
  var body = simplify(ast.body);
  var elseBody = simplify(ast.elseBody);
  if (body.type === 'empty' && elseBody.type === 'empty') {
    return _ast2.default.Empty;
  } else {
    return _ast2.default.If(simplify(ast.predicate), body, elseBody);
  }
};

var simplifyBody = function simplifyBody(ast) {
  var mapped = _lodash2.default.flatMap(ast.body, function (child) {
    var simplified = simplify(child);
    if (simplified.type === 'body') {
      return simplified.body;
    } else if (simplified.type === 'empty') {
      return [];
    } else {
      return [simplified];
    }
  });

  var firstReturn = _lodash2.default.findIndex(mapped, function (node) {
    return node.type === 'return';
  });
  var body = firstReturn >= 0 ? mapped.slice(0, firstReturn + 1) : mapped;
  if (body.length === 0) {
    return _ast2.default.Empty;
  } else if (body.length === 1) {
    return body[0];
  } else {
    return _ast2.default.Body.apply(_ast2.default, _toConsumableArray(body));
  }
};

var simplify = function simplify(ast) {
  switch (ast.type) {
    case 'assignment':
      return _ast2.default.Assignment(ast.variable, simplify(ast.value));
    case 'if':
      return simplifyIf(ast);
    case 'return':
      return ast;
    case 'body':
      return simplifyBody(ast);
    case 'for':
    case 'forin':
    case 'empty':
      return ast;
    case 'function1':
      return _ast2.default.Function1(ast.name, ast.argument, simplify(ast.body));
    case 'binop':
    default:
      return ast;
  }
};

exports.default = simplify;