'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _ast = require('./js/jsast/ast.js');

var _ast2 = _interopRequireDefault(_ast);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var INDENT = '  ';
var indent = function indent(line, depth) {
  return '' + _lodash2.default.repeat(INDENT, depth) + line;
};

var primitivePredicate = function primitivePredicate(type, symbol) {
  switch (type) {
    case 'integer':
      return 'typeof ' + symbol + ' === \'number\' && ' + symbol + ' % 1 === 0';
    case 'number':
      return 'typeof ' + symbol + ' === \'number\'';
    case 'string':
      return 'typeof ' + symbol + ' === \'string\'';
    case 'object':
      return symbol + ' && typeof ' + symbol + ' === \'object\' && !Array.isArray(' + symbol + ')';
    case 'array':
      return 'Array.isArray(' + symbol + ')';
    case 'boolean':
      return 'typeof ' + symbol + ' === \'boolean\'';
    case 'null':
      return symbol + ' === null';
    default:
      return 'false';
  }
};

// Wraps lines of code in a check such that it only executes when the value of
// `symbol` is the type
var typeCheck = function typeCheck(type, symbol, body) {
  return _ast2.default.If(primitivePredicate(type, symbol), body);
};

exports.default = {
  indent: indent,
  primitivePredicate: primitivePredicate,
  typeCheck: typeCheck
};